import React from 'react';
import PropTypes from 'prop-types';
import momentPropTypes from 'react-moment-proptypes';
import cx from 'classnames';
import moment from 'moment';

const propTypes = {
  id: PropTypes.number.isRequired,
  isInstant: PropTypes.bool,
  getStyle: PropTypes.func,
  eventStart: PropTypes.string.isRequired,
  eventEnd: PropTypes.string.isRequired,

  onDeleteClick: PropTypes.func,
  onInstantClick: PropTypes.func,
};

const defaultProps = {
  isInstant: false,
  getStyle() {},

  onDeleteClick() {},
  onInstantClick() {},
};

class Event extends React.PureComponent {
  constructor(props) {
    super(props);

    this.onDeleteClick = this.onDeleteClick.bind(this);
    this.onInstantClick = this.onInstantClick.bind(this);
  }

  shouldComponentUpdate(nextProps) {
    const { isInstant, eventStart, eventEnd } = this.props;
    return isInstant !== nextProps.isInstant
      || eventStart !== nextProps.eventStart
      || eventEnd !== nextProps.eventEnd;
  }

  onDeleteClick(e) {
    if (e) e.preventDefault();

    const { id, onDeleteClick } = this.props;
    onDeleteClick(id);
  }
  onInstantClick(e) {
    if (e) e.preventDefault();

    const { id, onInstantClick } = this.props;
    onInstantClick(id);
  }

  render() {
    const {
      getStyle,
      eventStart,
      eventEnd,
      isInstant,
    } = this.props;

    const className = cx('Event', {
      'Event--instant': isInstant,
    });

    return (
      <div
        className={className}
        style={getStyle()}
      >
        <div className="Event__action Event__instant" onClick={this.onInstantClick}>
          <span>
            <i className="fa fa-bolt" />
          </span>
        </div>
        <div className="Event__content">
          <div className="Event__container">
            <span>
              {moment(eventStart).format('LT')} - {moment(eventEnd).format('LT')}
            </span>
          </div>
        </div>
        <div className="Event__action Event__delete" onClick={this.onDeleteClick}>
          <span>
            <i className="fa fa-times" />
          </span>
        </div>
      </div>
    );
  }
}

Event.propTypes = propTypes;
Event.defaultProps = defaultProps;

export default Event;