import React from 'react';
import PropTypes from 'prop-types';
import momentPropTypes from 'react-moment-proptypes';
import moment from 'moment';
import Slot from './Slot';

const propTypes = {
  hourStart: momentPropTypes.momentObj.isRequired,
  hourEnd: momentPropTypes.momentObj.isRequired,
  slotDuration: momentPropTypes.momentDurationObj.isRequired,

  onSlotMouseEnter: PropTypes.func,
  onSlotMouseLeave: PropTypes.func,
  onSlotMouseClick: PropTypes.func,
  isSlotAssignable: PropTypes.func,
};

const defaultProps = {
  onSlotMouseEnter() {},
  onSlotMouseLeave() {},
  onSlotMouseClick() {},
  isSlotAssignable() { return true; },
};

class Hour extends React.Component {
  constructor(props) {
    super(props);

    this.onSlotMouseEnter = this.onSlotMouseEnter.bind(this);
    this.onSlotMouseLeave = this.onSlotMouseLeave.bind(this);
    this.onSlotMouseClick = this.onSlotMouseClick.bind(this);
  }

  onSlotMouseEnter(slotStart) {
    const { onSlotMouseEnter } = this.props;
    onSlotMouseEnter(slotStart);
  }
  onSlotMouseLeave() {
    const { onSlotMouseLeave } = this.props;
    onSlotMouseLeave();
  }
  onSlotMouseClick(slotStart) {
    const { onSlotMouseClick, slotDuration } = this.props;
    onSlotMouseClick(slotStart, moment(slotStart).add(slotDuration));
  }

  renderSlots() {
    const {
      slotDuration,
      hourStart,
      isSlotAssignable,
    } = this.props;
    const slotDurationInMinutes = slotDuration.minutes();
    const slots = [];

    for (let i = 0; i < (60 / slotDurationInMinutes); i += 1) {
      slots.push({
        key: i,
        start: moment(hourStart).startOf('hour').add(i * slotDurationInMinutes, 'minutes'),
        end: moment(hourStart).startOf('hour').add(((i + 1) * slotDurationInMinutes), 'minutes'),
      });
    }

    return (
      <div className="Hour__slots">
        {slots.map(slot => (
          <Slot
            key={slot.key}
            slotStart={slot.start}
            slotEnd={slot.end}
            onSlotMouseEnter={() => this.onSlotMouseEnter(slot.start)}
            onSlotMouseLeave={() => this.onSlotMouseLeave(slot.start)}
            onSlotMouseClick={() => this.onSlotMouseClick(slot.start)}
            isAssignable={() => isSlotAssignable(slot.start, slot.end)}
          />
        ))}
      </div>
    );
  }
  render() {
    return (
      <div className="Hour">
        {this.renderSlots()}
      </div>
    );
  }
}

Hour.propTypes = propTypes;
Hour.defaultProps = defaultProps;

export default Hour;