import React from 'react';
import PropTypes from 'prop-types';
import momentPropTypes from 'react-moment-proptypes';

const propTypes = {
  slotStart: momentPropTypes.momentObj.isRequired,
  slotEnd: momentPropTypes.momentObj.isRequired,

  onSlotMouseEnter: PropTypes.func,
  onSlotMouseLeave: PropTypes.func,
  onSlotMouseClick: PropTypes.func,
  isAssignable: PropTypes.func,
};

const defaultProps = {
  onSlotMouseEnter() {},
  onSlotMouseLeave() {},
  onSlotMouseClick() {},
  isAssignable() { return true; },
};

class Slot extends React.Component {
  constructor(props) {
    super(props);

    this.onSlotMouseEnter = this.onSlotMouseEnter.bind(this);
    this.onSlotMouseLeave = this.onSlotMouseLeave.bind(this);
    this.onSlotMouseClick = this.onSlotMouseClick.bind(this);
  }
  onSlotMouseEnter(e) {
    if (e) e.preventDefault();

    const {
      onSlotMouseEnter,
      isAssignable,
    } = this.props;

    if (isAssignable()) {
      onSlotMouseEnter();
    }
  }
  onSlotMouseLeave(e) {
    if (e) e.preventDefault();

    const {
      onSlotMouseLeave,
      isAssignable,
    } = this.props;

    if (isAssignable) {
      onSlotMouseLeave();
    }
  }
  onSlotMouseClick(e) {
    if (e) e.stopPropagation();

    const {
      onSlotMouseClick,
      isAssignable,
    } = this.props;

    if (isAssignable) {
      onSlotMouseClick();
    }
  }

  render() {
    return (
      <div
        className="Slot"
        onMouseEnter={(e) => { this.onSlotMouseEnter(e); }}
        onMouseLeave={(e) => { this.onSlotMouseLeave(e); }}
        onClick={(e) => { this.onSlotMouseClick(e); }}
      />
    );
  }
}

Slot.propTypes = propTypes;
Slot.defaultProps = defaultProps;

export default Slot;
