import React from 'react';
import PropTypes from 'prop-types';
import moment from 'moment';
import momentPropTypes from 'react-moment-proptypes';
import Hour from './Hour';

const propTypes = {
  day: momentPropTypes.momentObj.isRequired,
  slotDuration: momentPropTypes.momentDurationObj.isRequired,

  hourStart: PropTypes.number,
  hourEnd: PropTypes.number,

  onSlotMouseEnter: PropTypes.func,
  onSlotMouseLeave: PropTypes.func,
  onSlotMouseClick: PropTypes.func,
  isSlotAssignable: PropTypes.func,
};

const defaultProps = {
  hourStart: 0,
  hourEnd: 24,

  onSlotMouseEnter() {},
  onSlotMouseLeave() {},
  onSlotMouseClick() {},
  isSlotAssignable() { return true; },
};

class SlotsContainer extends React.Component {
  shouldComponentUpdate(nextProps) {
    const { day } = this.props;
    return !day.isSame(nextProps.day);
  }

  getHours() {
    const {
      hourStart,
      hourEnd,
    } = this.props;
    const hours = [];

    for (let i = hourStart; i < hourEnd; i += 1) {
      hours.push(i);
    }

    return hours;
  }

  render() {
    const {
      slotDuration,
      onSlotMouseEnter,
      onSlotMouseLeave,
      onSlotMouseClick,
      isSlotAssignable,
      day,
    } = this.props;

    const hours = this.getHours();

    return (
      <div className="SlotsContainer">
        {hours.map(hour => (
          <Hour
            key={hour}
            slotDuration={slotDuration}
            hourStart={moment(day).startOf('day').add(hour, 'hours')}
            hourEnd={moment(day).startOf('day').add(hour + 1, 'hours')}
            onSlotMouseEnter={onSlotMouseEnter}
            onSlotMouseLeave={onSlotMouseLeave}
            onSlotMouseClick={onSlotMouseClick}
            isSlotAssignable={isSlotAssignable}
          />
        ))}
      </div>
    );
  }
}

SlotsContainer.propTypes = propTypes;
SlotsContainer.defaultProps = defaultProps;

export default SlotsContainer;
